﻿namespace Ecng.Trading.Xaml
{
	using System;
	using System.Collections.Generic;
	using System.Collections.ObjectModel;
	using System.Windows;

	using AmCharts.Windows.Stock;
	using AmCharts.Windows.Stock.Data;

	using Ecng.Common;
	using Ecng.Trading.BusinessEntities;
	using Ecng.Collections;

	/// <summary>
	/// Графический компонент отображения графиков.
	/// </summary>
	[CLSCompliant(false)]
	public partial class ChartControl
	{
		private sealed class CustomChartIndicatorList<T> : BaseCollection<T>
			where T : class
		{
			private readonly ObservableCollection<T> _inner;
			private readonly Dictionary<DateTime, T> _dict = new Dictionary<DateTime, T>();

			public CustomChartIndicatorList( ObservableCollection<T> inner)
			{
				if (inner == null)
					throw new ArgumentNullException("inner");

				_inner = inner;
			}

			protected override void OnAdding(T item)
			{
				var time = GetTime(item);

				var old = _dict.TryGetValue(time);
				if (old != null)
					_inner.Remove(item);

				_inner.Add(item);
				_dict[time] = item;
				base.OnAdding(item);
			}

			protected override void OnRemoving(T item)
			{
				_inner.Remove(item);
				_dict.Remove(GetTime(item));

				base.OnRemoving(item);
			}

			protected override void OnInserting(int index, T item)
			{
				OnAdding(item);
				base.OnInserting(index, item);
			}

			protected override void OnClearing()
			{
				_inner.Clear();
				_dict.Clear();

				base.OnClearing();
			}

			private static DateTime GetTime(T item)
			{
				if (item == null)
					throw new ArgumentNullException("item");

				if (item is Candle)
					return item.To<Candle>().Time;
				else if (item is Order)
					return item.To<Order>().Time;
				else
					return item.To<CustomChartIndicator>().Time;
			}
		}

		/// <summary>
		/// Создать <see cref="ChartControl"/>.
		/// </summary>
		public ChartControl()
		{
			this.DataContext = this;

			var candlesSource = new ObservableCollection<Candle>();
			var ordersSource = new ObservableCollection<Order>();

			this.Candles = new CustomChartIndicatorList<Candle>(candlesSource);
			this.Orders = new CustomChartIndicatorList<Order>(ordersSource);

			InitializeComponent();

			_stockChart.DataSets[0].ItemsSource = candlesSource;
			_stockChart.DataSets[1].ItemsSource = ordersSource;
			_stockChart.EndDate = DateTime.Now;
			_stockChart.StartDate = _stockChart.EndDate - TimeSpan.FromTicks(TimeSpanEx.TicksPerYear);
		}

		/// <summary>
		/// Коллекция свечек, по которым строить график OHLC.
		/// </summary>
		public ICollection<Candle> Candles { get; private set; }

		/// <summary>
		/// Коллекция заявок, по которым рисуются сигналы на покупку и продажу.
		/// </summary>
		public ICollection<Order> Orders { get; private set; }

		/// <summary>
		/// Создать дополнительный тренд.
		/// </summary>
		/// <param name="title">Заголовок нового тренда.</param>
		/// <param name="type">Тип нового тренда.</param>
		/// <returns>Коллекция с данными, которую необходимо заполнять для перерисовки созданного тренда.</returns>
		[CLSCompliant(false)]
		public ICollection<CustomChartIndicator> CreateTrend(string title, GraphType type)
		{
			var source = new ObservableCollection<CustomChartIndicator>();

			var dataSet = new DataSet
			{
				Title = title,
				ShortTitle = title,
				ItemsSource = source,
				DateMemberPath = "Time",
				ValueMemberPath = "Value",
				ID = title,
			};

			_stockChart.DataSets.Add(dataSet);

			_mainChart.Graphs.Add(new Graph
			{
				DataSet = dataSet,
				GraphType = type,
				Style = (Style)_stockChart.Resources["sharedGraphSettings"],
			});

			return new CustomChartIndicatorList<CustomChartIndicator>(source);
		}
	}
}